require "greek_fire/version"
require "greek_fire/engine"
require "greek_fire/configuration"

module GreekFire
  class DuplicateMetricError < StandardError; end

  class << self
    attr_accessor :configuration

    def configure
      self.configuration ||= Configuration.new
      yield(configuration) if block_given?
      register_sidekiq_metrics if configuration.sidekiq_metrics && Object.const_defined?("Sidekiq")
    end

    private

    def register_sidekiq_metrics
      duplicate_metrics = []

      sidekiq_metrics.each do |metric|
        if metric_already_registered? metric[:name]
          duplicate_metrics << metric[:name]
        else
          GreekFire::Gauge.register metric[:name], description: metric[:description], labels: metric[:labels], &metric[:proc]
        end
      end

      raise DuplicateMetricError,
        "The following metrics have already been registered by default. Please remove them from your greek_fire.rb initializer: #{duplicate_metrics.join(", ")}" if duplicate_metrics.any?
    end

    def metric_already_registered?(metric_name)
      GreekFire::Metric.metrics.map(&:name).include? metric_name
    end

    def sidekiq_metrics
      [
        {
          name: "sidekiq_process_count",
          description: "How many sidekiq processes are running",
          proc: Proc.new { |labels| Sidekiq::ProcessSet.new.size }
        },
        {
          name: "sidekiq_job_latency",
          description: "How long are jobs enqueued",
          labels: { :queue => Proc.new { Sidekiq::Queue.all.map(&:name) }},
          proc: Proc.new { |labels| Sidekiq::Queue.new(labels[:queue]).latency }
        },
        {
          name: "sidekiq_job_count",
          description: "How many jobs are enqueued",
          labels: { :queue => Proc.new { Sidekiq::Queue.all.map(&:name) }},
          proc: Proc.new { |labels| Sidekiq::Queue.new(labels[:queue]).size }
        }
      ]
    end
  end
end
